@@ -1,42 +1,23 @@
 package Authen::Captcha;
 
 # $Source: /usr/local/cvs/Captcha/pm/Captcha.pm,v $ 
-# $Revision: 1.22 $
-# $Date: 2003/12/04 20:46:12 $
+# $Revision: 1.23 $
+# $Date: 2003/12/18 04:44:34 $
 # $Author: jmiller $ 
 # License: GNU General Public License Version 2 (see license.txt)
 
 use 5.00503;
 use strict;
 use GD;
-use Digest::MD5 qw(md5_hex);
+use String::Random qw(random_regex);
 use Carp;
 # these are used to find default images dir
 use File::Basename;
 use File::Spec;
 
-require Exporter;
-use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
-@ISA = qw(Exporter);
+use vars qw($VERSION);
 
-# Items to export into callers namespace by default. Note: do not export
-# names by default without a very good reason. Use EXPORT_OK instead.
-# Do not simply export all your public functions/methods/constants.
-
-# This allows declaration	use Authen::Captcha ':all';
-# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
-# will save memory.
-%EXPORT_TAGS = ( 'all' => [ qw(
-	
-) ] );
-
-@EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
-
-@EXPORT = qw(
-	
-);
-
-$VERSION = sprintf "%d.%03d", q$Revision: 1.22 $ =~ /(\d+)/g;
+$VERSION = '1.024';
 
 # get our file name, used to find the default images
 my $default_images_folder;
@@ -47,6 +28,8 @@ my $default_images_folder;
 	$default_images_folder = File::Spec->catdir(@this_dirs,'Captcha','images');
 }
 
+my $num_of_soundfile_versions = 10;
+
 # Preloaded methods go here.
 
 sub new
@@ -59,6 +42,8 @@ sub new
 	my %opts = @_;
 
 	# default character source images
+	my $type = defined($opts{type}) ? $opts{type} : 'image';
+	$self->type($type);
 	my $src_images = (defined($opts{images_folder}) && (-d $opts{images_folder}))
 	                 ? $opts{images_folder} : $default_images_folder;
 	$self->images_folder($src_images);
@@ -77,6 +62,9 @@ sub new
 	my $height = (defined($opts{height}) && ($opts{height} =~ /^\d+$/))
 	             ? $opts{height} : 35;
 	$self->height($height);
+	my $keep_failures = (defined($opts{keep_failures}) && $opts{keep_failures})
+	                    ? 1 : 0;
+	$self->keep_failures($keep_failures);
 	
 	# create a random seed if perl version less than 5.004
 	if ($] < 5.005)
@@ -87,6 +75,23 @@ sub new
 	return $self;
 }
 
+sub type
+{
+	ref(my $self = shift) or croak "instance variable needed";
+	if (@_)
+	{
+		if ($_[0] =~ /^(jpg|png|gif|image|picture)$/i)
+		{
+			$self->{_type} = 'image';
+		} elsif ($_[0] =~ /^(sound|snd|wav|mp3)$/i) {
+			$self->{_type} = 'sound';
+		}
+		return $self->{_type};
+	} else {
+		return $self->{_type};
+	}
+}
+
 sub debug
 {
 	ref(my $self = shift) or croak "instance variable needed";
@@ -99,6 +104,19 @@ sub debug
 	}
 }
 
+sub keep_failures
+{
+	ref(my $self = shift) or croak "instance variable needed";
+	if (@_)
+	{
+		croak "keep_failures must be a zero or one" unless ($_[0] =~ /^[01]$/);
+		$self->{_keep_failures} = $_[0];
+		return $self->{_keep_failures};
+	} else {
+		return $self->{_keep_failures};
+	}
+}
+
 sub expire 
 {
 	ref(my $self = shift) or croak "instance variable needed";
@@ -179,14 +197,15 @@ sub data_folder
 sub check_code 
 {
 	ref(my $self = shift) or croak "instance variable needed";
-	my ($code, $crypt) = @_;
+	my ($code, $token) = @_;
 
 	$code = lc($code);
 	
-	warn "$code  $crypt\n" if($self->debug() >= 2);
+	warn "$code  $token\n" if($self->debug() >= 2);
 
 	my $current_time = time;
-	my $return_value = 0;
+	# solution was not found in database (well, yet :)
+	my $return_value = -2;
 	my $database_file = File::Spec->catfile($self->data_folder(),"codes.txt");
 
 	# create database file if it doesn't already exist
@@ -196,12 +215,10 @@ sub check_code
 	# they could be confused with (o) and (l), so we swap them in
 	$code =~ tr/01/ol/;
 
-	my $md5 = md5_hex($code);
-	
 	# pull in current database
 	warn "Open File: $database_file\n" if($self->debug() >= 2);
+	$self->_get_exclusive_lock();
 	open (DATA, "<$database_file")  or die "Can't open File: $database_file\n";
-		flock DATA, 1;  # read lock
 		my @data=<DATA>;
 	close(DATA);
 	warn "Close File: $database_file\n" if($self->debug() >= 2);
@@ -210,65 +227,99 @@ sub check_code
 	# $new_data will hold the part of the database we want to keep and 
 	# write back out
 	my $new_data = "";
-	my $found;
 	foreach my $line (@data) 
 	{
 		$line =~ s/\n//;
-		my ($data_time,$data_code) = split(/::/,$line);
-		
-		my $png_file = File::Spec->catfile($self->output_folder(),$data_code . ".png");
-		if ($data_code eq $crypt)
+		my ($data_time,$data_token,$data_code) = $line =~ m/(^\d+)::([a-f0-9]{32})::(.*)$/
+			or next;
+
+		my $png_file = File::Spec->catfile($self->output_folder(),$data_token . ".png");
+		if ($data_token eq $token)
 		{
-			# the crypt was found in the database
+
+			# the token was found in the database
 			if (($current_time - $data_time) > $self->expire())
 			{ 
 				 warn "Crypt Found But Expired\n" if($self->debug() >= 2);
-				# the crypt was found but has expired
+				# the token was found but has expired
 				$return_value = -1;
 			} else {
-				warn "Match Crypt in File Crypt: $crypt\n" if($self->debug() >= 2);
-				$found = 1;
+				warn "Match Crypt in File Crypt: $token\n" if($self->debug() >= 2);
 			}
-			# remove the found crypt so it can't be used again
-			warn "Unlink File: " . $png_file . "\n" if($self->debug() >= 2);
-			unlink($png_file) or carp("Can't remove png file [$png_file]\n");
+
+			if ( ($data_code eq $code) && ($return_value != -1) )
+			{
+				warn "Match: " . $data_token . " And " . $token . "\n" if($self->debug() >= 2);
+				# solution was correct and was found in database - passed
+				$return_value = 1;
+			}
+
+			if ( $return_value < ($self->keep_failures() ? -1 : -2) )
+			{
+				warn "No Match: " . $data_token . " And " . $token . "\n" if($self->debug() >= 2);
+				# solution was wrong, not expired, and we're keeping failures
+				$new_data .= $line."\n";
+			} else {
+				# remove the found token so it can't be used again
+				warn "Unlink File: " . $png_file . "\n" if($self->debug() >= 2);
+				unlink($png_file) or carp("Can't remove png file [$png_file]\n");
+			}
+
+			if ( $return_value == -2 ) {
+				# incorrect solution
+				$return_value = -3;
+			}
+
 		} elsif (($current_time - $data_time) > $self->expire()) {
-			# removed expired crypt
+			# removed expired token
 			warn "Removing Expired Crypt File: " . $png_file ."\n" if($self->debug() >= 2);
 			unlink($png_file) or carp("Can't remove png file [$png_file]\n");
 		} else {
-			# crypt not found or expired, keep it
+			# token not found or expired, keep it
 			$new_data .= $line."\n";
 		}
 	}
 
-	if ($md5 eq $crypt)
-	{
-		warn "Match: " . $md5 . " And " . $crypt . "\n" if($self->debug() >= 2);
-		# solution was correct
-		if ($found)
-		{
-			# solution was correct and was found in database - passed
-			$return_value = 1;
-		} elsif (!$return_value) {
-			# solution was not found in database
-			$return_value = -2;
-		}
-	} else {
-		warn "No Match: " . $md5 . " And " . $crypt . "\n" if($self->debug() >= 2);
-		# incorrect solution
-		$return_value = -3;
-	}
-
 	# update database
 	open(DATA,">$database_file")  or die "Can't open File: $database_file\n";
-		flock DATA, 2; # write lock 
 		print DATA $new_data;
 	close(DATA);
+	$self->_release_lock();
 	
 	return $return_value;
 }
 
+
+sub _open_lock_file {
+	my $self = shift;
+	my $file_name = shift;
+	open(LOCK, ">>$file_name") or die "Error opening lockfile $file_name: $!\n";
+}
+
+sub _get_shared_lock {
+	my $self = shift;
+	my $lock_file_name = File::Spec->catfile($self->data_folder(),"codes.lock");
+	$self->_open_lock_file($lock_file_name);
+
+	# shared lock
+	flock(LOCK, 1) or die "Error locking lockfile in shared mode: $!\n";
+}
+
+sub _get_exclusive_lock {
+	my $self = shift;
+	my $lock_file_name = File::Spec->catfile($self->data_folder(),"codes.lock");
+	$self->_open_lock_file($lock_file_name);
+
+	# exclusive lock
+	flock(LOCK, 2) or die "Error locking lockfile exclusively: $!\n";
+}
+
+sub _release_lock {
+	my $self = shift;
+	flock(LOCK, 8) or die "Error unlocking lockfile: $!\n";
+	close(LOCK);
+}
+
 sub _touch_file
 {
 	ref(my $self = shift) or croak "instance variable needed";
@@ -281,21 +332,11 @@ sub _touch_file
 	}
 }
 
-sub generate_code 
+sub generate_random_string
 {
 	ref(my $self = shift) or croak "instance variable needed";
-	my ($length) = @_;
+	my $length = shift;
 
-	my $database_file = File::Spec->catfile($self->data_folder(),'codes.txt');
-	my $im_width = $self->width();
-
-	# set a variable with the current time
-	my $current_time = time;
-
-	# create a new image and color
-	my $im = new GD::Image(($im_width * $length),$self->height());
-	my $black = $im->colorAllocate(0,0,0);
-	
 	# generate a new code
 	my $code = "";
 	for(my $i=0; $i < $length; $i++)
@@ -311,8 +352,69 @@ sub generate_code
 		$char = chr($char);
 		$code .= $char;
 	}
-	my $md5 = md5_hex($code);
+	return $code;
+}
+
+sub _save_code
+{
+	ref(my $self = shift) or croak "instance variable needed";
+	my $code = shift;
+	my $token = shift;
+
+	my $database_file = File::Spec->catfile($self->data_folder(),'codes.txt');
+
+	# set a variable with the current time
+	my $current_time = time;
+
+	# create database file if it doesn't already exist
+	$self->_touch_file($database_file);
+
+	# clean expired codes and images
+	$self->_get_exclusive_lock();
+
+	open (DATA, "<$database_file")  or die "Can't open File: $database_file\n";
+		my @data=<DATA>;
+	close(DATA);
 	
+	my $new_data = "";
+	foreach my $line (@data) 
+	{
+		$line =~ s/\n//;
+		my ($data_time,$data_token,$data_code) = $line =~ m/(^\d+)::([a-f0-9]{32})::(.*)$/
+			or next;
+		if ( (($current_time - $data_time) > ($self->expire())) ||
+		     ($data_token  eq $token) )
+		{	# remove expired captcha, or a dup
+			my $png_file = File::Spec->catfile($self->output_folder(),$data_token . ".png");
+			unlink($png_file) or carp("Can't remove png file [$png_file]\n");
+		} else {
+			$new_data .= $line."\n";
+		}
+	}
+	
+	# save the code to database
+	warn "open File: $database_file\n" if($self->debug() >= 2);
+	open(DATA,">$database_file")  or die "Can't open File: $database_file\n";
+		warn "-->>" . $new_data . "\n" if($self->debug() >= 2);
+		warn "-->>" . $current_time . "::" . $token."::".$code."\n" if($self->debug() >= 2);
+		print DATA $new_data;
+		print DATA $current_time."::".$token."::".$code."\n";
+	close(DATA);
+	$self->_release_lock();
+	warn "Close File: $database_file\n" if($self->debug() >= 2);
+}
+
+sub create_image_file
+{
+	ref(my $self = shift) or croak "instance variable needed";
+	my $code = shift;
+
+	my $length = length($code);
+	my $im_width = $self->width();
+	# create a new image and color
+	my $im = new GD::Image(($im_width * $length),$self->height());
+	my $black = $im->colorAllocate(0,0,0);
+
 	# copy the character images into the code graphic
 	for(my $i=0; $i < $length; $i++)
 	{
@@ -363,55 +465,77 @@ sub generate_code
 	# add a border
 	$im->rectangle(0,0,((($length)*($self->width()))-1),(($self->height())-1),$black);
 
-	# create database file if it doesn't already exist
-	$self->_touch_file($database_file);
-
-	# clean expired codes and images
-	open (DATA, "<$database_file")  or die "Can't open File: $database_file\n";
-		flock DATA, 1;  # read lock
-		my @data=<DATA>;
-	close(DATA);
-	
-	my $new_data = "";
-	foreach my $line (@data) 
-	{
-		$line =~ s/\n//;
-		my ($data_time,$data_code) = split(/::/,$line);
-		if ( (($current_time - $data_time) > ($self->expire())) ||
-		     ($data_code  eq $md5) )
-		{	# remove expired captcha, or a dup
-			my $png_file = File::Spec->catfile($self->output_folder(),$data_code . ".png");
-			unlink($png_file) or carp("Can't remove png file [$png_file]\n");
-		} else {
-			$new_data .= $line."\n";
-		}
-	}
-	
-	# save the code to database
-	warn "open File: $database_file\n" if($self->debug() >= 2);
-	open(DATA,">$database_file")  or die "Can't open File: $database_file\n";
-		flock DATA, 2; # write lock
-		warn "-->>" . $new_data . "\n" if($self->debug() >= 2);
-		warn "-->>" . $current_time . "::" . $md5."\n" if($self->debug() >= 2);
-		print DATA $new_data;
-		print DATA $current_time."::".$md5."\n";
-	close(DATA);
-	warn "Close File: $database_file\n" if($self->debug() >= 2);
-	
 	# save the image to file
-	my $output_file = File::Spec->catfile($self->output_folder(),$md5 . ".png");
 	my $png_data = $im->png;
 
-	warn "Open File: $output_file\n" if($self->debug() >= 2);
-	open (FILE,">$output_file") or die "Can't open File: $output_file \n";
+	return \$png_data;
+}
+
+sub create_sound_file
+{
+	ref(my $self = shift) or croak "instance variable needed";
+	my $code = shift;
+	my $length = length($code);
+
+	my @chars = split('',$code);
+	my $snd_file;
+	local $/; # input record separator. So we can slurp the data.
+	# get a random voice speaking the code
+	foreach my $char (@chars)
+	{
+		my $voice = int(rand $num_of_soundfile_versions) + 1;
+		my $src_name = File::Spec->catfile($self->images_folder(),$voice, $char . ".wav");
+		warn "Open File: $src_name\n" if($self->debug() >= 2);
+		open (FILE,"< $src_name") or die "Can't open File: $src_name\n";
+			flock FILE, 1; # read lock
+			binmode FILE;
+			$snd_file .= <FILE>;
+		close FILE;
+		warn "Close File: $src_name\n" if($self->debug() >= 2);
+	}
+	return \$snd_file;
+}
+
+sub _save_file
+{
+	ref(my $self = shift) or croak "instance variable needed";
+	my $file_ref = shift;
+	my $file_name = shift;
+
+	warn "Open File: $file_name\n" if($self->debug() >= 2);
+	open (FILE,">$file_name") or die "Can't open File: $file_name \n";
 		flock FILE, 2; # write lock
 		binmode FILE;
-		print FILE $png_data;
+		print FILE $$file_ref;
 	close FILE;
-	warn "Close File: $output_file\n" if($self->debug() >= 2);
-	
-	# return crypt (md5)... or, if they want it, the code as well.
-	return wantarray ? ($md5,$code) : $md5;
+	warn "Close File: $file_name\n" if($self->debug() >= 2);
+}
+
+sub generate_code 
+{
+	ref(my $self = shift) or croak "instance variable needed";
+	my $length = shift;
+
+	my $code = $self->generate_random_string($length);
+	my $token = random_regex('[a-f0-9]{32}');
+
+	my ($captcha_data_ref,$output_filename);
+	if ($self->type() eq 'image')
+	{
+		$captcha_data_ref = $self->create_image_file($code);
+		$output_filename = File::Spec->catfile($self->output_folder(),$token . ".png");
+	} elsif ($self->type() eq 'sound') {
+		$captcha_data_ref = $self->create_sound_file($code);
+		$output_filename = File::Spec->catfile($self->output_folder(),$token . ".wav");
+	} else {
+		croak "invalid captcha type [" . $self->type() . "]";
+	}
+
+	$self->_save_file($captcha_data_ref,$output_filename);
+	$self->_save_code($code,$token);
+
+	# return token (token)... or, if they want it, the code as well.
+	return wantarray ? ($token,$code) : $token;
 }
 
 sub version
@@ -437,7 +561,7 @@ Authen::Captcha - Perl extension for creating captcha's to verify the human elem
   # set the data_folder. contains flatfile db to maintain state
   $captcha->data_folder('/some/folder');
 
-  # set directory to hold publicly accessable images
+  # set directory to hold publicly accessible images
   $captcha->output_folder('/some/http/folder');
 
   # Alternitively, any of the methods to set variables may also be
@@ -448,24 +572,28 @@ Authen::Captcha - Perl extension for creating captcha's to verify the human elem
     output_folder => '/some/http/folder',
     );
 
-  # create a captcha. Image filename is "$md5sum.png"
-  my $md5sum = $captcha->generate_code($number_of_characters);
+  # create a captcha. Image filename is "$token.png"
+  my $token = $captcha->generate_code($number_of_characters);
 
   # check for a valid submitted captcha
   #   $code is the submitted letter combination guess from the user
-  #   $md5sum is the submitted md5sum from the user (that we gave them)
-  my $results = $captcha->check_code($code,$md5sum);
+  #   $token is the submitted token from the user (that we gave them)
+  my $results = $captcha->check_code($code,$token);
   # $results will be one of:
   #          1 : Passed
   #          0 : Code not checked (file error)
   #         -1 : Failed: code expired
   #         -2 : Failed: invalid code (not in database)
-  #         -3 : Failed: invalid code (code does not match crypt)
+  #         -3 : Failed: invalid code (code does not match token)
   ##############
 
+=head1 WARNING
+
+The captcha produced by this module is rather weak compared to other modules available. You are advised to update to L<GD::SecurityImage>, which provides API-compatible interface in L<GD::SecurityImage::AC> module.
+
 =head1 ABSTRACT
 
-Authen::Captcha provides an object oriented interface to captcha file creations.  Captcha stands for Completely Automated Public Turning test to tell Computers and Humans Apart. A Captcha is a program that can generate and grade tests that:
+Authen::Captcha provides an object oriented interface to captcha file creations.  Captcha stands for Completely Automated Public Turing test to tell Computers and Humans Apart. A Captcha is a program that can generate and grade tests that:
 
     - most humans can pass.
     - current computer programs can't pass
@@ -527,31 +655,39 @@ See the method descriptions for more detail on what they mean.
    width =>  25, # optional. default 25
    height => 35, # optional. default 35
    images_folder => '/some/folder', # optional. default to lib dir
+   keep_failures => 0, # optional, defaults to 0(false)
    debug => 0, # optional. default 0
 
 =back
 
-=item C<$md5sum = $captcha-E<gt>generate_code( $number_of_characters );>
+=item C<$token = $captcha-E<gt>generate_code( $number_of_characters );>
 
-Creates a captcha. Image filename is "$md5sum.png"
+Creates a captcha. Image filename is "$token.png"
 
 It can also be called in array context to retrieve the string of characters used to generate the captcha (the string the user is expected to respond with). This is useful for debugging.
 ex.
 
-C<($md5sum,$chars) = $captcha-E<gt>generate_code( $number_of_characters );>
+C<($token,$chars) = $captcha-E<gt>generate_code( $number_of_characters );>
 
-=item C<$results = $captcha-E<gt>check_code($code,$md5sum);>
+=item C<$results = $captcha-E<gt>check_code($code,$token);>
 
 check for a valid submitted captcha
+
 $code is the submitted letter combination guess from the user
-$md5sum is the submitted md5sum from the user (that we gave them)
+
+$token is the submitted token from the user (that we gave them)
+
+If the $code and $token are correct, the image file and database entry will be removed.
+
+If the $token matches one in the database, and "keep_failures" is false (the default), the image file and database entry will be removed to avoid repeated attempts on the same captcha.
+
 $results will be one of:
 
     1 : Passed
     0 : Code not checked (file error)
    -1 : Failed: code expired
    -2 : Failed: invalid code (not in database)
-   -3 : Failed: invalid code (code does not match crypt)
+   -3 : Failed: invalid code (code does not match token)
 
 =back
 
@@ -561,12 +697,12 @@ $results will be one of:
 
 =item C<$captcha-E<gt>data_folder( '/some/folder' );>
 
-Required. Sets the directory to hold the flatfile database that will be used to store the current non-expired valid captcha md5sum's.
-Must be writable by the process running the script (usually the web server user, which is usually either "apache" or "http"), but should not be accessable to the end user.
+Required. Sets the directory to hold the flatfile database that will be used to store the current non-expired valid captcha tokens.
+Must be writable by the process running the script (usually the web server user, which is usually either "apache" or "http"), but should not be accessible to the end user.
 
 =item C<$captcha-E<gt>output_folder( '/some/folder' );>
 
-Required. Sets the directory to hold the generated Captcha image files. This is usually a web accessable directory so that the user can view the images in here, but it doesn't have to be web accessable (you could be attaching the images to an e-mail for some verification, or some other Captcha implementation).
+Required. Sets the directory to hold the generated Captcha image files. This is usually a web accessible directory so that the user can view the images in here, but it doesn't have to be web accessible (you could be attaching the images to an e-mail for some verification, or some other Captcha implementation).
 Must be writable by the process running the script (usually the web server user, which is usually either "apache" or "http").
 
 =item C<$captcha-E<gt>images_folder( '/some/folder' );>
@@ -586,6 +722,10 @@ Optional. Number of pixels high for the character graphics. Defaults to 25.
 
 Optional. Number of pixels wide for the character graphics. Defaults to 35.
 
+=item C<$captcha-E<gt>keep_failures( [0|1] );>
+
+Optional. Defaults to zero. This option controls whether or not the captcha will remain valid after a failed attempt. By default, we only allow one attempt to solve it. This greatly reduces the possibility that a bot could brute force a correct answer. Change it at your own risk.
+
 =item C<$captcha-E<gt>debug( [0|1|2] );>
 
 Optional. 
@@ -613,12 +753,16 @@ Josh I. Miller, E<lt>jmiller@purifieddata.netE<gt>
 
 First Productions, Inc. created the cgi-script distributed under the GPL which was used as the basis for this module. Much work has gone into making this more robust, and suitable for other applications, but much of the origonal code remains.
 
+Fixes were reported and contributed by various people, see Changes file for a complete list.
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2003, First Productions, Inc. (FIRSTPRODUCTIONS HUMAN TEST 1.0)
 
 Copyright 2003 by Seth Jackson
 
+Copyright 2012 by Paolo Rossi, Lubomir Rintel, Chris Dunlop, Gert Schepens and Ernesto Hernández-Novich
+
 This library is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. (see license.txt).
 
 This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
